<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <link rel="stylesheet" href="/static/style.css" type="text/css" />
</head>
<body>
    <div align="center" class="document" style="font-size: 100%;">
      <div class="related">
        <ul> 
          <li class="right" style="margin-right: 10px">{{ loginurl }} - {{ who }}</i></a> |</li>
          <li><a href="http://jsonbot.org/index.html" onclick='window.open("http://jsonbot.org/index.html"); return false'><b>DOCS</b></a></li>
          <li>-</li>
          <li><a href="/demo" title="demo"><b>demo</b></a></li>
        </ul>
      </div> 
    <div class="footer" align="center" id="topper_div"></div><br>
    <div class="body" align="center"><h1><b>{{ appname }}</b></h1></div>
    <div class="body" align="center" style="font-size: 100%;">
        <h2><a href="/login_required">Click here to login with google</a></h2>
        <<h3>We only use your email adress</h3>
    </div>
</body>
</html>
