"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

from sleekxmpp.test.mocksocket import TestSocket
from sleekxmpp.test.livesocket import TestLiveSocket
from sleekxmpp.test.sleektest import *
